/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.ws.cxf.easybeans;

import org.apache.cxf.Bus;
import org.apache.cxf.jaxws.EndpointImpl;
import org.apache.cxf.jaxws.JaxWsServerFactoryBean;
import org.ow2.easybeans.container.session.stateless.StatelessSessionFactory;

/**
 * Builds an EasyBeans Endpoint based on the JAX WS endpoint provided by CXF.
 * @author Florent Benoit
 */
public class EasyBeansEndpoint extends EndpointImpl {

    /**
     * Stateless Factory class used for the endpoint.
     */
    private StatelessSessionFactory factory;

    /**
     * Builds the EasyBeans endpoint with the given parameters.
     * @param factory the EasyBeans factory.
     * @param bus the CXF Bus
     * @param serverFactory the JAX WS server factory.
     */
    public EasyBeansEndpoint(final StatelessSessionFactory factory,
                             final Bus bus,
                             final JaxWsServerFactoryBean serverFactory) {
        super(bus, factory.getBeanClass(), serverFactory);
        this.factory = factory;
    }

    /**
     * Gets the class of the implementor.
     * @return the class of the implementor object
     */
    @Override
    public Class<?> getImplementorClass() {
        return this.factory.getBeanClass();
    }
}
